<?php
class ModelExtensionShippingBaikalsr extends Model {
	
	public function install() {
		$this->db->query("
			CREATE TABLE IF NOT EXISTS `" . DB_PREFIX . "baikalsr_shipment_order` (
				`id` INT(11) NOT NULL AUTO_INCREMENT,
				`order_id` int(11) NOT NULL,
				`data` text NOT NULL,
				`shipping_id` int(11) DEFAULT NULL,
				`hash` varchar(36) DEFAULT NULL,
				`tracking_number` varchar(36) DEFAULT NULL,
				`status_id` int(11),
				`status_name` text,
				`message` text,
				`shipping_cost` DECIMAL( 10, 2 ) DEFAULT NULL,	
				`date_added` DATETIME NOT NULL,
				`modified` DATETIME NOT NULL,				
				PRIMARY KEY `id` (`id`)
			) ENGINE=MyISAM DEFAULT CHARSET=utf8 COLLATE=utf8_general_ci;
		");

		$this->db->query("
			CREATE TABLE IF NOT EXISTS `" . DB_PREFIX . "baikalsr_session` (
			  `baikalsr_session_id` INT(11) NOT NULL AUTO_INCREMENT,		  
			  `session_id` varchar(32) NOT NULL,
			  `data` text NOT NULL,
			  `ip` varchar(40) NOT NULL,
			  `date_added` DATETIME NOT NULL,
			  PRIMARY KEY `baikalsr_session_id` (`baikalsr_session_id`)
			) ENGINE=MyISAM DEFAULT COLLATE=utf8_general_ci;
		");
	}
	
	public function uninstall() {
		$this->db->query("DROP TABLE IF EXISTS `" . DB_PREFIX . "baikalsr_shipment_order`;");
		$this->db->query("DROP TABLE IF EXISTS `" . DB_PREFIX . "baikalsr_session`;");
	}
	
	public function getOrders($data = array()) {
		$sql = "SELECT o.order_id, CONCAT(o.firstname, ' ', o.lastname) AS customer, os.shipping_id, os.status_name AS shipping_status_name, o.shipping_method, o.shipping_city, o.shipping_code, (SELECT ot.value FROM " . DB_PREFIX . "order_total ot WHERE ot.order_id = o.order_id AND ot.code = 'shipping') AS shipping_cost, os.shipping_cost AS new_shipping_cost, o.total, o.currency_code, o.currency_value, o.date_added, o.date_modified FROM `" . DB_PREFIX . "order` o LEFT JOIN `" . DB_PREFIX . "baikalsr_shipment_order` os ON (o.order_id = os.order_id)";

		$sql .= " WHERE o.shipping_code LIKE 'baikalsr%'";

		//baiklsr shipping filter
		if (!empty($data['filter_order_id'])) {
			$sql .= " AND o.order_id = '" . (int)$data['filter_order_id'] . "'";
		}

		if (!empty($data['filter_shipping_id'])) {
			$sql .= " AND os.shipping_id = '" . (int)$data['filter_shipping_id'] . "'";
		}
		
		if (!empty($data['filter_customer'])) {
			$sql .= " AND CONCAT(o.firstname, ' ', o.lastname) LIKE '%" . $this->db->escape($data['filter_customer']) . "%'";
		}

		if (!empty($data['filter_shipping_city'])) {
			$sql .= " AND o.shipping_city LIKE '%" . $this->db->escape($data['filter_shipping_city']) . "%'";
		}

		$sort_data = array(
			'o.order_id',
			'o.date_added',
			'customer',
			'o.shipping_city',
			'os.shipping_id',
			'o.shipping_code',
			'shipping_cost',
			'o.total'
		);

		if (isset($data['sort']) && in_array($data['sort'], $sort_data)) {
			$sql .= " ORDER BY " . $data['sort'];
		} else {
			$sql .= " ORDER BY o.order_id";
		}

		if (isset($data['order']) && ($data['order'] == 'DESC')) {
			$sql .= " DESC";
		} else {
			$sql .= " ASC";
		}

		if (isset($data['start']) || isset($data['limit'])) {
			if ($data['start'] < 0) {
				$data['start'] = 0;
			}

			if ($data['limit'] < 1) {
				$data['limit'] = 20;
			}

			$sql .= " LIMIT " . (int)$data['start'] . "," . (int)$data['limit'];
		}

		$query = $this->db->query($sql);

		return $query->rows;
	}	

	public function getTotalOrders($data = array()) {
		$sql = "SELECT COUNT(*) AS total FROM `" . DB_PREFIX . "order` o LEFT JOIN `" . DB_PREFIX . "baikalsr_shipment_order` os ON (o.order_id = os.order_id)";

		$sql .= " WHERE o.shipping_code LIKE 'baikalsr%'";
		
		//baiklsr shipping filter
		if (!empty($data['filter_order_id'])) {
			$sql .= " AND o.order_id = '" . (int)$data['filter_order_id'] . "'";
		}

		if (!empty($data['filter_shipping_id'])) {
			$sql .= " AND os.shipping_id = '" . (int)$data['filter_shipping_id'] . "'";
		}
		
		if (!empty($data['filter_customer'])) {
			$sql .= " AND CONCAT(o.firstname, ' ', o.lastname) LIKE '%" . $this->db->escape($data['filter_customer']) . "%'";
		}

		if (!empty($data['filter_shipping_city'])) {
			$sql .= " AND o.shipping_city LIKE '%" . $this->db->escape($data['filter_shipping_city']) . "%'";
		}

		$query = $this->db->query($sql);

		return $query->row['total'];
	}
	
	public function updateShipmentOrder($form_data) {
        $values = array();
		
        $insert_data = array();
        $insert_data['id'] = (int)$form_data['id'];
		$insert_data['order_id'] = (int)$form_data['order_id'];
		$insert_data['data'] = "'" . $this->db->escape(json_encode($form_data['data'])). "'";
		$insert_data['shipping_id'] = "'" . $this->db->escape($form_data['shipping_id']) . "'";
		$insert_data['hash'] = "'" . $this->db->escape($form_data['hash']) . "'";
        $insert_data['tracking_number'] = "'" . $this->db->escape($form_data['tracking_number']) . "'";
        $insert_data['status_id'] = (int)$form_data['status_id'];
        $insert_data['status_name'] = "'" . $this->db->escape($form_data['status_name']) . "'";
        $insert_data['message'] = "'" . $this->db->escape($form_data['message']) . "'";	
        $insert_data['shipping_cost'] = (float)$form_data['shipping_cost'];
        $insert_data['date_added'] = "NOW()";
        $insert_data['modified'] = 'NOW()';
		
        $values[] = "(" . implode(",", $insert_data) . ")";

        $sql = "INSERT INTO `" . DB_PREFIX . "baikalsr_shipment_order` (`id`, `order_id`, `data`, `shipping_id`, `hash`, `tracking_number`, `status_id`, `status_name`, `message`, `shipping_cost`, `date_added`, `modified`) VALUES " . implode(',', $values) . " ON DUPLICATE KEY UPDATE `data`=VALUES(`data`), `shipping_id`=VALUES(`shipping_id`), `hash`=VALUES(`hash`), `tracking_number`=VALUES(`tracking_number`), `status_id`=VALUES(`status_id`), `status_name`=VALUES(`status_name`), `message`= VALUES(`message`), `shipping_cost`= VALUES(`shipping_cost`), `modified`=NOW()";

        $this->db->query($sql);
    }
	
	public function getContragents($apiKey = null, $apiTypeTest = null)
	{
		$contragents = array();
		$this->load->library('baikalsrtools');
		$response = $this->baikalsrtools->getContragents($apiKey, $apiTypeTest);
		foreach($response as $data) {
			if (isset($data["guid"]) && !empty($data["guid"]) && isset($data["name"]) && !empty($data["name"]))
				$contragents[] = array('id' => $data['guid'], 'name' => $data['name']);
		}
		return $contragents;
	}	
	
	public function getTerminals($parent_guid, $terminal_type, $apiKey = null, $apiTypeTest = null)
	{
		$terminals = array();
		if($parent_guid){
			$this->load->library('baikalsrtools');
			$arAffiliateData = $this->baikalsrtools->getAffiliateData($parent_guid, $apiKey, $apiTypeTest);
			if (isset($arAffiliateData["terminals"]) && !empty($arAffiliateData["terminals"])) {
				foreach($arAffiliateData["terminals"] as $terminal) {
					if ($terminal[$terminal_type]) {
						$terminals[] = array('id' => $terminal['id'], 'name' => $terminal['name']." (".$this->baikalsrtools->htmlspecialcharsBack($terminal["address"]).")");
					}
				}
			}
		}
		return $terminals;
	}
	
    public function getOpf()
    {
		$opf = array();
		$this->load->library('baikalsrtools');
		$response = $this->baikalsrtools->getPartnerTypes();
		foreach($response as $data) {
			if(isset($data["name"]) && !empty($data["name"])
			&&	isset($data["legalForm"]) && !empty($data["legalForm"]))
				$opf[] = $data;
		}

        return $opf;
    }
	
	public function getShipmentOrder($order_id) {

		$qry = $this->db->query("SELECT * FROM `" . DB_PREFIX . "baikalsr_shipment_order` WHERE `order_id` = '" . (int)$order_id . "' LIMIT 1");

		if ($qry->num_rows) {
			$result = $qry->row;
			$result['data'] = json_decode($qry->row['data'], true);
			return $result;
		} else {
			return [];
		}
	}
	
	public function getSessionDataByIP($ip) {

		$qry = $this->db->query("SELECT * FROM `" . DB_PREFIX . "baikalsr_session` WHERE `ip` = '" . $ip . "' ORDER BY date_added DESC LIMIT 1");

		if ($qry->num_rows) {
			$result = $qry->row;
			return json_decode($qry->row['data'], true);
		} else {
			return false;
		}
	}	
}