<?php
class ModelExtensionShippingBaikalsr extends Model {

	function getQuote($address) {
		$this->load->library('baikalsrtools');
		$this->load->language('extension/shipping/baikalsr');

		$query = $this->db->query("SELECT * FROM " . DB_PREFIX . "zone_to_geo_zone WHERE geo_zone_id = '" . (int)$this->config->get('shipping_baikalsr_geo_zone_id') . "' AND country_id = '" . (int)$address['country_id'] . "' AND (zone_id = '" . (int)$address['zone_id'] . "' OR zone_id = '0')");

		if (!$this->config->get('shipping_baikalsr_geo_zone_id')) {
			$status = true;
		} elseif ($query->num_rows) {
			$status = true;
		} else {
			$status = false;
		}

		$error = false;

		$quote_data = array();

		if ($status) {
			$products = array();
            foreach ($this->cart->getProducts() as $product) {
                if ($product['shipping']) {
                    $products[] = $product;
                }
			}
			$arCity = $this->getFiasFromAddress($address);
			$courierCost = $this->calculateConcrete("courier", $arCity, $products);
			if($courierCost){			
				$quoteText = $this->currency->format($courierCost["total"], $this->session->data['currency']);	
				if(isset($courierCost["transit"]["int"]) && isset($courierCost["transit"]["day"]))
					$quoteText .=  ' (' . $this->language->get('text_term') . ': ' . $courierCost["transit"]["int"] . ' ' . $courierCost["transit"]["day"] . ')';
				$quote_data['baikalsr'] = array(
					'code'         => 'baikalsr.baikalsr',
					'title'        => $this->language->get('text_to_address'),
					'cost'         => $courierCost["total"],
					'tax_class_id' => 0,
					'text'         => $quoteText
				);
			}
			$pickupCost = $this->calculateConcrete("pickup", $arCity, $products);
			if($pickupCost){
				$arrTerminals = $this->getTerminals($arCity);
				$quoteText = $this->currency->format($pickupCost["total"], $this->session->data['currency']);		
				if(isset($pickupCost["transit"]["int"]) && isset($pickupCost["transit"]["day"]))
					$quoteText .=  ' (' .$this->language->get('text_term') . ': ' . $pickupCost["transit"]["int"] . ' ' . $pickupCost["transit"]["day"] . ')';
				if(!empty($arrTerminals)){
					$quoteText .= '<div class="form-group">';
					$quoteText .= '<span class="control-label" for="select-destination-terminal">' . $this->language->get('text_terminal') . '</span>';
					$quoteText .= '<select id="select-destination-terminal" class="form-control">';
					foreach ($arrTerminals as $terminalData) {
						$quoteText .= '<option value="'.$terminalData['id'].'">'.$terminalData['name'].'</option>';
					}
					$quoteText .= '</select>';
					$quoteText .= '</div>';
					$quoteText .= '<script>';
					$quoteText .= '$("#select-destination-terminal").change(function() {';
					$quoteText .= '	var guid_1c = $(this).children(":selected").attr("value");';
					$quoteText .= '	(async function(){';
					$quoteText .= '		var a = await fetch("?route=extension/shipping/baikalsr/iffi&guid_1c="+guid_1c);';
					$quoteText .= '		var b = await a.text();';
					$quoteText .= '	})();';
					$quoteText .= '});';
					$quoteText .= '</script>';
				}
				$quote_data['baikalsr2'] = array(
					'code'         => 'baikalsr.baikalsr2',
					'title'        => $this->language->get('text_to_terminal'),
					'cost'         => $pickupCost["total"],
					'tax_class_id' => 0,
					'text'         => $quoteText
				);
			}
		}

		$method_data = array();

		if (!empty($quote_data)) {
			$method_data = array(
				'code'       => 'baikalsr',
				'title'      => $this->language->get('text_title'),
				'quote'      => $quote_data,
				'sort_order' => $this->config->get('shipping_baikalsr_sort_order'),
				'error'      => $error
			);
		}

		return $method_data;
	}
	
	protected function getFiasFromAddress($address)
	{
		$arCity = [];
		if(!empty($address['city'])){
			$arCity = $this->baikalsrtools->getCityByName($address['city']);
		}	
		if(empty($arCity) && !empty($address['zone'])){
			$arCity = $this->baikalsrtools->getCityByName($this->baikalsrtools->untranslit($address['zone']));
		}
		return $arCity;
	}
	
	protected function calculateConcrete($profile, $arCity, $products)
    {
		$arrPrice = [];
        $data = [];
		if(!empty($arCity) && isset($arCity['guid']) && !empty($arCity['guid'])) {
			$data['baikalsrGuid'] = $arCity['guid'];
			$data['baikalsrAffiliateGuid'] = $arCity['affiliate']['guid'] ? $arCity['affiliate']['guid'] : $arCity['guid'];
			$data['basketPrice'] = $this->getBasketPrice($products);    
			$data['goods'] = $this->getDimensionsAndWeight($products);
			if ($this->isDataValid($data)) {	
				$calcResult = $this->baikalsrtools->calculate($profile, $data);
				if (isset($calcResult["total"]) && !empty($calcResult["total"])) $arrPrice = $calcResult;
			}
       	}   
        return $arrPrice;
    }
	
	protected function getTerminals($arCity)
    {
		$arrTerminals = [];
		if(!empty($arCity) && isset($arCity['guid']) && !empty($arCity['guid'])) {
			$response = $this->baikalsrtools->getAffiliateData($arCity['affiliate']['guid'] ? $arCity['affiliate']['guid'] : $arCity['guid']);
			if (isset($response["terminals"]) && !empty($response["terminals"])) {
				foreach($response["terminals"] as $terminal) {
					if ($terminal["finish"]) {
						$terminal["address"] = $this->baikalsrtools->htmlspecialcharsBack($terminal["address"]);
						$arrTerminals[] = array('id' => $terminal['id'], 'name' => $terminal['name']." (".$terminal["address"].")");
					}
				}
			}
		}
		         
        return $arrTerminals;
    }
	
	public function getDimensionsAndWeight($products) {
        $arDimensions = array();
		$defaultForItem = $this->config->get('shipping_baikalsr_default_for_item');
		$defLength = $this->baikalsrtools->convertLength($this->config->get('shipping_baikalsr_length'), $this->config->get('shipping_baikalsr_length_class_id'));
		$defWidth = $this->baikalsrtools->convertLength($this->config->get('shipping_baikalsr_width'), $this->config->get('shipping_baikalsr_length_class_id'));
		$defHeight = $this->baikalsrtools->convertLength($this->config->get('shipping_baikalsr_height'), $this->config->get('shipping_baikalsr_length_class_id'));
		$defWeight = $this->baikalsrtools->convertWeight($this->config->get('shipping_baikalsr_weight'), $this->config->get('shipping_baikalsr_weight_class_id'));
        $totalVolume1 = 0;
		$totalWeight1 = 0;
        $totalVolume2 = 0;
		$totalWeight2 = 0;
        $maxLength = 0;
        $maxWidth = 0;
        $maxHeight = 0;
        //max dimension
        foreach ($products as $product) {
			if(!empty((float) $product['weight']) && !empty($product['weight_class_id'])){
				$totalWeight1 += $product['quantity'] * $this->baikalsrtools->convertWeight($product['weight'], $product['weight_class_id']);	
			}else{
				if($defaultForItem)
					$totalWeight2 += $product['quantity'] * $defWeight;
				else
					$totalWeight2 = $defWeight;
			}				
			if(!empty((float) $product['length']) && !empty((float) $product['width']) && !empty((float) $product['height']) && !empty($product['length_class_id'])){
				$length = $this->baikalsrtools->convertLength($product['length'], $product['length_class_id']);
				$width = $this->baikalsrtools->convertLength($product['width'], $product['length_class_id']);
				$height = $this->baikalsrtools->convertLength($product['height'], $product['length_class_id']);
				$totalVolume1 += $product['quantity'] * $length * $width * $height;
			}else{
				$length = $defLength;
				$width = $defWidth;
				$height = $defHeight;
				if($defaultForItem)
					$totalVolume2 += $product['quantity'] * $defLength * $defWidth * $defHeight;
				else
					$totalVolume2 = $defLength * $defWidth * $defHeight;
			}
			if($length > $maxLength) $maxLength = $length;
			if($width > $maxWidth) $maxWidth = $width;
			if($height > $maxHeight) $maxHeight = $height;
        }
		$arDimensions['length']= $maxLength;
        $arDimensions['width'] = $maxWidth;
        $arDimensions['height'] = $maxHeight;
		$arDimensions['volume'] = $totalVolume1 + $totalVolume2;
		$arDimensions['weight'] = $totalWeight1 + $totalWeight2;
        return $arDimensions;
    }		
	
	public function getBasketPrice($products){
        $totalPrice = 0;
        foreach($products as $product){
			$totalPrice += $this->tax->calculate($product['total'], $product['tax_class_id'], $this->config->get('config_tax'));
        }
        return $totalPrice;
    }
	
    protected function isDataValid($data)
    {
        return
            (
                $data['basketPrice'] > 0
            ) && (
                !empty($this->config->get('shipping_baikalsr_departure_guid'))
            ) && (
                isset($data['baikalsrGuid'])  &&
                !empty($data["baikalsrGuid"])
            ) && (
                isset($data['baikalsrAffiliateGuid'])  &&
                !empty($data["baikalsrAffiliateGuid"])
            ) ;
    }	
}