<?php

class BaikalsrTools
{
	private $session;
    private $url;
    private $config;
    private $log;
    private $customer;
    private $currency;
    private $registry;
	
    const API_URL_TEST = "https://test-api.baikalsr.ru/";
    const API_URL = "https://api.baikalsr.ru/";
	
	public function __construct($registry) {
        $this->session = $registry->get('session');
        $this->url = $registry->get('url');
        $this->config = $registry->get('config');
        $this->log = $registry->get('log');
        $this->customer = $registry->get('customer');
        $this->currency = $registry->get('currency');
        $this->length = $registry->get('length');
        $this->weight = $registry->get('weight');
        $this->registry = $registry;
    }

    private function apiGetRequest($apiUrl, $apiKey = null, $apiTypeTest = null)
    {
		if(!isset($apiTypeTest))
			$apiTypeTest = $this->config->get('shipping_baikalsr_test');
        if ($apiTypeTest) {
            if(!isset($apiKey))
				$apiKey = $this->config->get('shipping_baikalsr_authkeytest');
            $apiUrl = self::API_URL_TEST.$apiUrl;
        } else {
			if(!isset($apiKey))
				$apiKey = $this->config->get('shipping_baikalsr_authkey');
            $apiUrl = self::API_URL.$apiUrl;
        }
        if ($apiKey) {
            if ($objCurl = curl_init()) {
                curl_setopt_array($objCurl, array(
                    CURLOPT_URL => $apiUrl,
                    CURLOPT_HTTPAUTH => CURLAUTH_BASIC,
                    CURLOPT_USERPWD => $apiKey.':',
                    CURLOPT_RETURNTRANSFER => true,
                    CURLOPT_SSL_VERIFYPEER => false,
                    CURLOPT_TIMEOUT => 60,
                    CURLOPT_HTTPHEADER => array(
                        'Content-Type: application/json'
                    )
                ));
                $result = curl_exec($objCurl);
                if ($result === false) {
                    $error = 'CURL error: ' . curl_error($objCurl);
                    return array("Errors" => [$error]);
                }
                curl_close($objCurl);
            }
            return json_decode($result, true);
        }
        return array();
    }

    private function apiPostRequest($apiUrl, $params = [], $apiKey = null, $apiTypeTest = null)
    {
        if(!isset($apiTypeTest))
			$apiTypeTest = $this->config->get('shipping_baikalsr_test');
        if ($apiTypeTest) {
            if(!isset($apiKey))
				$apiKey = $this->config->get('shipping_baikalsr_authkeytest');
            $apiUrl = self::API_URL_TEST.$apiUrl;
        } else {
			if(!isset($apiKey))
				$apiKey = $this->config->get('shipping_baikalsr_authkey');
            $apiUrl = self::API_URL.$apiUrl;
        }
        if ($apiKey) {
            if ($objCurl = curl_init()) {
                curl_setopt_array($objCurl, array(
                    CURLOPT_URL => $apiUrl,
                    CURLOPT_HTTPAUTH => CURLAUTH_BASIC,
                    CURLOPT_USERPWD => $apiKey.':',
                    CURLOPT_RETURNTRANSFER => true,
                    CURLOPT_POST => true,
                    CURLOPT_POSTFIELDS => json_encode($params),
                    CURLOPT_SSL_VERIFYPEER => false,
                    CURLOPT_TIMEOUT => 60,
                    CURLOPT_HTTPHEADER => array(
                        'Content-Type: application/json'
                    )
                ));
                $result = curl_exec($objCurl);
                if ($result === false) {
                    $error = 'CURL error: ' . curl_error($objCurl);
                    return array("Errors" => [$error]);
                }
                curl_close($objCurl);
            }
            return json_decode($result, true);
        }
        return array();
    }

    public function getAffiliateData($guid, $apiKey = null, $apiTypeTest = null)
    {
        $affiliateData = [];
        $apiUrl = "v1/affiliate/";
        if ($guid) {
            $apiUrl .= $guid;

            $affiliateData = $this->apiGetRequest($apiUrl, $apiKey, $apiTypeTest);
        }

        return $affiliateData;
    }
	
    public function getCitiesByName($city, $region = "", $apiKey = null, $apiTypeTest = null)
    {
        $cityData = [];
        $apiUrl = "v2/fias/cities?text=";
        if ($city) {
            $apiUrl .= $city;
            if ($region) $apiUrl .= " ".$region;

            $allCitiesData = $this->apiGetRequest($apiUrl, $apiKey, $apiTypeTest);
        }

        return $allCitiesData;
    }
	
    public function getCityByName($city, $region = "", $apiKey = null, $apiTypeTest = null)
    {
        $cityData = [];
        $apiUrl = "v2/fias/cities?text=";
        if ($city) {
            $apiUrl .= $city;
            if ($region) $apiUrl .= " ".$region;

            $allCityData = $this->apiGetRequest($apiUrl, $apiKey, $apiTypeTest);
            if(!empty($allCityData) && isset($allCityData[0]))
				if ($allCityData[0]) {
					$cityData = $allCityData[0];
				}
        }

        return $cityData;
    }

    public function getCityByGuid($guid, $apiKey = null, $apiTypeTest = null)
    {
        $cityData = [];
        $apiUrl = "v2/fias/cities?guid=";
        if ($guid) {
            $apiUrl .= $guid;

            $allCityData = $this->apiGetRequest($apiUrl, $apiKey, $apiTypeTest);
            if ($allCityData[0]) {
                $cityData = $allCityData[0];
            }
        }

        return $cityData;
    }

    public function getStreetsByName($cityGuid, $street = "", $apiKey = null, $apiTypeTest = null)
    {
        $allStreetsData = [];
        $apiUrl = "v2/fias/streets?guid=".$cityGuid."&text=";
        if ($street) {
            $apiUrl .= $street;

            $allStreetsData = $this->apiGetRequest($apiUrl, $apiKey, $apiTypeTest);
        }

        return $allStreetsData;
    }

    public function getHousesByName($streetGuid, $house = "", $apiKey = null, $apiTypeTest = null)
    {
        $allHousesData = [];
        $apiUrl = "v2/fias/objects?guid=".$streetGuid."&h=";
        if ($house) {
            $apiUrl .= $house;

            $allHousesData = $this->apiGetRequest($apiUrl, $apiKey, $apiTypeTest);
        }

        return $allHousesData;
    }

    public function getCargoTypes($apiKey = null, $apiTypeTest = null)
    {
        $cargoTypes = [];
        $apiUrl = "v2/directory/cargotype";
        $cargoTypes = $this->apiGetRequest($apiUrl, $apiKey, $apiTypeTest);

        return $cargoTypes;
    }

    public function getServices($guid, $apiKey = null, $apiTypeTest = null)
    {
        $services = [];
        if ($guid) {
            $apiUrl = "v2/directory/services/".$guid;
            $services = $this->apiGetRequest($apiUrl, $apiKey, $apiTypeTest);
        }

        return $services;
    }

    public function getPartnerTypes($apiKey = null, $apiTypeTest = null)
    {
        $partnerTypes = [];
        $apiUrl = "v2/directory/partnertype";
        $partnerTypes = $this->apiGetRequest($apiUrl, $apiKey, $apiTypeTest);

        return $partnerTypes;
    }

    public function getContragents($apiKey = null, $apiTypeTest = null)
    {
        $contragents = [];
        $apiUrl = "v2/contragent/list";
        $contragents = $this->apiPostRequest($apiUrl, [], $apiKey, $apiTypeTest);

        return $contragents;
    }

    public function getDateOptions($guid)
    {
        $intervals = [];
        if ($guid) {
            $apiUrl = "v1/calculator/dateoptions";
            $params = [
                "guid" => $guid
            ];
            $intervals = $this->apiPostRequest($apiUrl, $params);
        }

        return $intervals;
    }

    public function getIntervals($guid, $apiKey = null, $apiTypeTest = null)
    {
        $intervals = [];
        if ($guid) {
            $apiUrl = "v2/directory/deliveryintervals/".$guid;
            $intervals = $this->apiGetRequest($apiUrl, $apiKey, $apiTypeTest);
        }

        return $intervals;
    }

    public function calculate($profile, $preorderData)
    {
        $apiUrl = "v2/calculator";

        $result = [];

		$departureGuid = $this->config->get('shipping_baikalsr_departure_guid') ? $this->config->get('shipping_baikalsr_departure_guid') : "";
		$departurePickup = $this->config->get('shipping_baikalsr_departure_from_address') ? "address" : "terminal";
        $gabarites = $preorderData["goods"];

        $destinationGuid = $preorderData["baikalsrGuid"];
        if ($profile == "pickup") {
            $destinationGuid = $preorderData["baikalsrAffiliateGuid"];
        }
		if(isset($gabarites["volume"]) && !empty($gabarites["volume"]))
			$volume = $gabarites["volume"];
		else
			$volume = $gabarites["length"] * $gabarites["width"] * $gabarites["height"];
		
        $params = [
            "Departure" => [
                "CityGuid" => $departureGuid,
            ],
            "Destination" => [
                "CityGuid" => $destinationGuid,
            ],
            "Cargo" => [
                "SummaryCargo" => [
                    "Length" => $gabarites["length"],
                    "Width" => $gabarites["width"],
                    "Height" => $gabarites["height"],
                    "Volume" => $volume,
                    "Weight" => $gabarites["weight"],
                    "Units" => 1,
                    //"Oversized" => 1,
                    "EstimatedCost" => $preorderData["basketPrice"] ? $preorderData["basketPrice"] : 0
                ]
            ],
            //"Preference" => [
            //    "AuthKey" => $apiKey,
            //"PartnerGUID" => ""
            //]
        ];
        if ($departurePickup == "address") {
            $street = $this->config->get('shipping_baikalsr_departure_street') ? $this->config->get('shipping_baikalsr_departure_street') : "";
            $streetGuid = $this->config->get('shipping_baikalsr_departure_street_guid') ? $this->config->get('shipping_baikalsr_departure_street_guid') : "";
            $house = $this->config->get('shipping_baikalsr_departure_house') ? $this->config->get('shipping_baikalsr_departure_house') : "";
            $housing = $this->config->get('shipping_baikalsr_departure_housing') ? $this->config->get('shipping_baikalsr_departure_housing') : "";
            $building = $this->config->get('shipping_baikalsr_departure_building') ? $this->config->get('shipping_baikalsr_departure_building') : "";
            $params["Departure"]["PickUp"] = [
                "Street" => $streetGuid,
                "House" => $house,
                "Housing" => $housing,
                "Building" => $building,
                "Date" => date("Y-m-d", strtotime("+1 day"))."T".date("00:00:00"),
                "TimeFrom" => "09:00",
                "TimeTo" => "18:00",
                "Services" => [3232]
            ];
            if ($this->config->get('shipping_baikalsr_departure_prr')) {
                $params["Departure"]["PickUp"]["prr"] = [
                    "floor" => $this->config->get('shipping_baikalsr_departure_floor'),
                    "lift" => $this->config->get('shipping_baikalsr_departure_lift') ? 1 : 0
                ];
            }
        }
        if ($profile == "courier") {
            $params["Destination"]["Delivery"] = [
                "Street" => "",
                "House" => "",
                "Housing" => "",
                "Building" => "",
                "TimeFrom" => "09:00",
                "TimeTo" => "18:00"
            ];
            if (isset($preorderData["street_guid"]) && !empty($preorderData["street_guid"])) {
                $params["Destination"]["Delivery"] = [
                    "Street" => $preorderData["street_guid"],
                    "House" => $preorderData["house"],
                    "Housing" => $preorderData["housing"],
                    "Building" => $preorderData["building"],
                    "TimeFrom" => "09:00",
                    "TimeTo" => "18:00"
                ];
            }

        }
		
		$packing = $this->config->get('shipping_baikalsr_packing');
		if($packing > 0){
			$fields[] = array("id"=>$packing, "value"=>1);
			$params["Cargo"]["SummaryCargo"]["servicesObject"][] = array("id"=>$packing, "fields"=>$fields);
		}

        /*
        file_put_contents($_SERVER["DOCUMENT_ROOT"]."/ifscr/calc_log3.txt", $prF);
        $prF = print_r($profile, true);
        file_put_contents($_SERVER["DOCUMENT_ROOT"]."/ifscr/calc_log3.txt", $prF, FILE_APPEND);*/

        $result = $this->apiPostRequest($apiUrl, $params);

        //file_put_contents($_SERVER["DOCUMENT_ROOT"]."/ifscr/calc_log3.txt", $prF, FILE_APPEND);

        return $result;
    }

    public function calculateOrder($profile, $preorderData)
    {
        $apiUrl = "v2/calculator";

        $result = [];

        $departurePickup = $preorderData["sendFromType"];

        $volume = $preorderData["gabarites"]["LENGTH"] * $preorderData["gabarites"]["WIDTH"] * $preorderData["gabarites"]["HEIGHT"];

        $params = [
            "Departure" => [
                "CityGuid" => $preorderData["departure"],
            ],
            "Destination" => [
                "CityGuid" => $preorderData["cityTo"],
            ],
            "Cargo" => [
                "SummaryCargo" => [
                    "Length" => $preorderData["gabarites"]["LENGTH"],
                    "Width" => $preorderData["gabarites"]["WIDTH"],
                    "Height" => $preorderData["gabarites"]["HEIGHT"],
                    "Volume" => $volume,
                    "Weight" => $preorderData["gabarites"]["WEIGHT"],
                    "Units" => 1,
                    "EstimatedCost" => $preorderData["cargo_price"] ? $preorderData["cargo_price"] : 0
                ]
            ],
            //"Preference" => [
            //    "AuthKey" => $apiKey,
            //"PartnerGUID" => ""
            //]
        ];
        if ($departurePickup == "address") {
            $params["Departure"]["PickUp"] = [
                "Street" => $preorderData["sender_street_guid"],
                "House" => $preorderData["sender_house"],
                "Housing" => $preorderData["sender_housing"],
                "Building" => $preorderData["sender_building"],
                "Date" => date("Y-m-d", strtotime($preorderData["sender_date"]))."T".date("00:00:00"),
                "TimeFrom" => "09:00",
                "TimeTo" => "18:00",
                "Services" => [3232]
            ];
            if ($preorderData["sender_prr"] == "Y") {
                $params["Departure"]["PickUp"]["prr"] = [
                    "floor" => $preorderData["sender_floor"],
                    "lift" => $preorderData["sender_lift"] == "Y" ? 1 : 0
                ];
            }
        }
        if ($profile == "courier" && $preorderData["street_guid"]) {
            $params["Destination"]["Delivery"] = [
                "Street" => $preorderData["street_guid"],
                "House" => $preorderData["house"],
                "Housing" => $preorderData["housing"],
                "Building" => $preorderData["building"],
                "TimeFrom" => "09:00",
                "TimeTo" => "18:00"
            ];
        }
        if ($preorderData["PACKING"]) {
            $params["Cargo"]["SummaryCargo"]["Services"] = $preorderData["PACKING"];
        }

        $result = $this->apiPostRequest($apiUrl, $params);

        return $result;
    }

    public function order($orderData)
    {
        $apiUrl = "v2/order";

        $result = [];

        //$volume = $orderData["gabarites"]["LENGTH"] * $orderData["gabarites"]["WIDTH"] * $orderData["gabarites"]["HEIGHT"];
        //list($opfId, $legalForm) = explode("_", $orderData["opf"]);
        $params = [
            "Author" => [
                "FirstName" => $orderData["author_firstname"],
                "MiddleName" => $orderData["author_middlename"],
                "LastName" => $orderData["author_lastname"],
                "Email" => $orderData["consignor_email"],
                "Phone" => $orderData["consignor_phone"],
                "Comment" => trim("ID заказа на сайте ".$orderData["order_id"])
            ],
            "Departure" => [
                "CityGuid" => $orderData["departure_guid"],
                /*
                "PickUp" => [
                    "StreetGuid" => "177e037d-a06d-4f8a-9333-fd534dd34c5d",
                    "house" => "23",
                    "housing" => "",
                    "building" => "",
                    "apartment" => "11",
                    "comment" => "звонить в звонок",
                    "Date" => "2020-12-31T00:00:00",
                    "TimeFrom" => "10:00",
                    "TimeTo" => "18:00",
                    "Services" => [
                        [
                            "id" => 1,
                            "PayerID" => "Consignor",
                            "FormOfPayment" => 2
                        ]
                    ],
                    "PayerID" => "Consignor",
                    "FormOfPayment" => 2
                ]*/
            ],
            "OrdersList" => [
                [
                    "Destination" => [
                        /*"Consignee" => [
                            "guid" => "",
                            "LegalForm" => $legalForm,
                            "Type" => $opfId,
                            "Name" => $orderData["company"],
                            "FirstField" => $orderData["inn"],
                            "SecondField" => $orderData["kpp"],
                            "Phone" => $orderData["phone"],
                            "Email" => $orderData["email"],
                            "ContactPerson" => $orderData["name"]
                        ],*/
                        "CityGuid" => $orderData["destination_guid"],
                    ],
                    "SummaryCargo" => [
                        "Length" => $orderData["length"],
                        "Width" => $orderData["width"],
                        "Height" => $orderData["height"],
                        "Volume" => $orderData["volume"],
                        "Weight" => $orderData["weight"],
                        "Units" => 1,
                        "Type" => $orderData["cargotype_id"],
                        "Insurance" => [
                            "EstimatedCost" => $orderData["estimated_cost"],
                            "PayerID" => "Consignor",
                            "FormOfPayment" => 2
                        ]
                    ],
                    "PayerID" => "Consignor",
                    "FormOfPayment" => 2
                ]
            ],
            /*"Payers" => [
                [
                    "PayerID" => "Payer1",
                    "GUID" => "4f86a22c-4758-4859-a227-a79c9ef500ff",
                    "LegalForm" => 1,
                    "Type" => 234,
                    "Name" => "ООО Юпитер",
                    "FirstField" => "1231231230",
                    "SecondField" => "123123001",
                    "Phone" => "+79513579513",
                    "Email" => "payer@mail.ru",
                    "ContactPerson" => "Эдуард"
                ]
            ]*/
        ];
        if ($orderData["consignor_contragent_id"]) {
            $params["Departure"]["Consignor"] = [
                "guid" => $orderData["consignor_contragent_id"],
                "Phone" => $orderData["consignor_phone"],
                "Email" => $orderData["consignor_email"],
                "ContactPerson" => $orderData["consignor_fio"]
            ];
        } /*else {
            list($opfId, $legalForm) = explode("_", $orderData["sender_opf"]);
            $params["Departure"]["Consignor"] = [
                "guid" => "",
                "LegalForm" => $legalForm,
                "Type" => $opfId,
                "Name" => $orderData["sender_company"],
                "FirstField" => $orderData["sender_inn"],
                "SecondField" => $orderData["sender_kpp"],
                "Phone" => $orderData["sender_phone"],
                "Email" => $orderData["sender_email"],
                "ContactPerson" => $orderData["sender_name"]
            ];
        }*/
        if ($orderData["departure_from_address"]) {
            $params["Departure"]["PickUp"] = [
                "StreetGuid" => $orderData["departure_street_guid"],
                "house" => $orderData["departure_house"],
                "housing" => $orderData["departure_housing"],
                "building" => $orderData["departure_building"],
                "apartment" => $orderData["departure_apartment"],
                "comment" => $orderData["departure_comment"],
                "Date" => date("Y-m-d", strtotime("+1 day"))."T00:00:00",//"2020-12-31T00:00:00",
                /*"TimeFrom" => "10:00",
                "TimeTo" => "18:00",
                "Services" => [
                    [
                        "id" => 1,
                        "PayerID" => "Consignor",
                        "FormOfPayment" => 2
                    ]
                ],*/
                "PayerID" => "Consignor",
                "FormOfPayment" => 2
            ];
            if (isset($orderData["departure_prr"])) {
                $params["Departure"]["PickUp"]["prr"] = [
                    "floor" => $orderData["departure_floor"],
                    "lift" => isset($orderData["departure_lift"]) ? true : false
                ];
            }
        } else {
            $params["Departure"]["Terminal"] = $orderData["departure_terminal_id"];
        }
        if ($orderData["consignee_physical_person"]) {
            $params["OrdersList"][0]["Destination"]["Consignee"] = [
                "guid" => "",
                "LegalForm" => 2,
                "Type" => $orderData["consignee_physical_document_id"],
                "Name" => $orderData["consignee_fio"],
                "FirstField" => $orderData["consignee_seria"],
                "SecondField" => $orderData["consignee_number"],
                "Phone" => $orderData["consignee_phone"],
                "Email" => $orderData["consignee_email"]
            ];
        } else {
			$legalForm = 1;
			if($orderData["consignee_legal_opf_id"] == 9)
				$legalForm = 4;
			elseif($orderData["consignee_legal_opf_id"] == 2309)
				$legalForm = 3;
            $params["OrdersList"][0]["Destination"]["Consignee"] = [
                "guid" => "",
                "LegalForm" => $legalForm,
                "Type" => $orderData["consignee_legal_opf_id"],
                "Name" => $orderData["consignee_company"],
                "FirstField" => $orderData["consignee_inn"],
                "SecondField" => $orderData["consignee_kpp"],
                "Phone" => $orderData["consignee_phone"],
                "Email" => $orderData["consignee_email"],
                "ContactPerson" => $orderData["consignee_fio"]
            ];
        }
        if ($orderData["destination_to_address"]) {
            $params["OrdersList"][0]["Destination"]["Delivery"] = [
                "StreetGuid" => $orderData["destination_street_guid"],
                "House" => $orderData["destination_house"],
                "Housing" => $orderData["destination_housing"],
                "Building" => $orderData["destination_building"],
                "Apartment" => $orderData["destination_apartment"],
                "CustomAddress" => "",
                "Comment" => $orderData["destination_comment"],
                /*
                "TimeFrom" => "10:00",
                "TimeTo" => "18:00",
                "Services" => [
                    [
                        "id" => 1,
                        "PayerID" => "Consignor",
                        "FormOfPayment" => 2
                    ]
                ],*/
                "PayerID" => "Consignor",
                "FormOfPayment" => 2
            ];
			if (isset($orderData["destination_prr"])) {
                $params["OrdersList"][0]["Destination"]["Delivery"]["prr"] = [
                    "floor" => $orderData["destination_floor"],
                    "lift" => isset($orderData["destination_lift"])? true : false
                ];
            }
        } else {
            $params["OrdersList"][0]["Destination"]["Terminal"] = $orderData["destination_terminal_id"];
        }
        if ($orderData["packing"]) {
            //foreach($orderData["PACKING"] as $pack) {
				$fields = [];
				$fields[] = array("id"=>$orderData["packing"], "value"=>1);
                $params["OrdersList"][0]["SummaryCargo"]["Services"][] = [
                    "id" => $orderData["packing"],
					"fields" => $fields,
                    "PayerID" => "Consignor",
                    "FormOfPayment" => 2
                ];
            //}
        }

        $result = $this->apiPostRequest($apiUrl, $params);

        return $result;
    }

    public function getOrderData($orderId)
    {
        $orderData = [];
        if ($orderId) {
            $apiUrl = "v2/order/detail";
            $orderData = $this->apiPostRequest($apiUrl, ["id" => $orderId]);
        }

        return $orderData;
    }
	
	public function convertLength($length, $lengthClassId){
		$length = (float) $length;
        $length_code = strtolower($this->length->getUnit($lengthClassId));
		if ($length_code == 'дюйм' || $length_code == 'in') {
			$length = $length/39;
		}else if ($length_code == 'мм' || $length_code == 'mm') {
			$length = $length/1000;
		}else if ($length_code == 'см' || $length_code == 'cm') {
			$length = $length/100;
		}
		
        return $length;
    }	
	
	public function convertWeight($weight, $weightClassId){
		$weight = (float) $weight;
        $weight_code = strtolower($this->weight->getUnit($weightClassId));

		if ($weight_code == 'г' || $weight_code == 'g') {
			$weight = $weight/1000;
		}

		if ($weight_code == 'lb') {
			$weight = $weight/2.20460000;
		}

		if ($weight_code == 'oz') {
			$weight = $weight/35.274;
		}
		
        return $weight;
    }	
	
	public function htmlspecialcharsBack($html_text)
	{
		return html_entity_decode($html_text);
	}	
	
	public function untranslit($textlat = null) {
		$cyr = array(
		'Абакан', 'Агинское', 'Анадырь', 'Архангельск', 'Астрахань', 'Барнаул', 'Белгород', 'Биробиджан', 'blagoveshchensk', 'Брянск', 'Чебоксары', 'Челябинск', 'Черкесск', 
		'Чита', 'Дудинка', 'Элиста', 'Горно-Алтайск', 'Грозный', 'Иркутск', 'Иваново', 'Ижевск', 'Калининград', 'Калуна', 'Краснодар', 'Казань', 'Кемерово', 'Хабаровск', 'Ханты-Мансийск',
		'Кострома', 'Краснодар', 'Красноярск', 'Кудымкар', 'Курган', 'Курск', 'Кызыл', 'Липетск', 'Магадан', 'Махачкала', 'Майкоп', 'Москва', 'Мурманск', 'Нальчик', 'Нарьян-Мар', 
		'Назрань', 'Нижний Новгород', 'Великий Новгород', 'Новосибирск', 'Омск', 'Орёл', 'Оренбург', 'Палана', 'Пенза', 'Пермь', 'Петропавловск-Камчатский', 'Петрозаводск', 'Псков', 
		'Ростов-на-Дону', 'Рязань', 'Салехард', 'Самара', 'Саранск', 'Саратов', 'Смоленск', 'Санкт-Петербург', 'Ставрополь', 'Сыктывкар', 'Тамбов', 'Томск', 'Тула', 'Тура', 'Тверь', 'Тюмень', 
		'Уфа', 'Ульяновск', 'Улан-Удэ', 'Усть-Ордынский', 'Владикавказ', 'Владимир', 'Владивосток', 'Волгоград', 'Вологда', 'Воронеж', 'Киров', 'Якутск', 'Ярославль', 'Екатеринбург', 
		'Йошкар-Ола');
		$lat = array(
		'abakan', 'aginskoye', 'anadyr', 'arkahangelsk', 'astrakhan', 'barnaul', 'belgorod', 'birobidzhan', 'blagoveshchensk', 'bryansk', 'cheboksary', 'chelyabinsk', 'cherkessk', 
		'chita', 'dudinka', 'elista', 'gorno-altaysk', 'groznyy', 'irkutsk', 'ivanovo', 'izhevsk', 'kalinigrad', 'kaluga', 'kasnodar', 'kazan', 'kemerovo', 'khabarovsk', 'khanty-mansiysk',
		'kostroma', 'krasnodar', 'krasnoyarsk', 'kudymkar', 'kurgan', 'kursk', 'kyzyl', 'lipetsk', 'magadan', 'makhachkala', 'maykop', 'moscow', 'murmansk', 'nalchik', 'naryan mar', 
		'nazran', 'nizhniy novgorod', 'novgorod', 'novosibirsk', 'omsk', 'orel', 'orenburg', 'palana', 'penza', 'perm', 'petropavlovsk-kamchatskiy', 'petrozavodsk', 'pskov', 
		'rostov-na-donu', 'ryazan', 'salekhard', 'samara', 'saransk', 'saratov', 'smolensk', 'st. petersburg', 'stavropol', 'syktyvkar', 'tambov', 'tomsk', 'tula', 'tura', 'tver', 'tyumen', 
		'ufa', 'ul\'yanovsk', 'ulan-ude', 'ust\'-ordynskiy', 'vladikavkaz', 'vladimir', 'vladivostok', 'volgograd', 'vologda', 'voronezh', 'vyatka', 'yakutsk', 'yaroslavl', 'yekaterinburg', 
		'yoshkar-ola');
		return str_replace($lat, $cyr, strtolower($textlat));
	}
}